package reflex.display
{
	import flash.display.DisplayObject;
	import flash.display.Shape;
	import flash.events.Event;
	
	import reflex.binding.Bind;
	import reflex.data.IPosition;
	import reflex.data.IScroll;
	import reflex.data.Position;
	import reflex.events.PropertyEvent;
	import reflex.measurement.resolveHeight;
	import reflex.measurement.resolveWidth;
	import reflex.metadata.resolveCommitProperties;
	
	[DefaultProperty("container")]
	public class ScrollDisplay extends Display
	{
		
		private var _horizontal:IPosition;
		private var _vertical:IPosition;
		private var _container:DisplayObject
		
		private var shape:Shape; // temporary
		
		[Bindable(event="horizontalChange")]
		public function get horizontal():IPosition { return _horizontal; }
		public function set horizontal(value:IPosition):void {
			if(_horizontal == value) {
				return;
			}
			PropertyEvent.dispatchChange(this, "horizontal", _horizontal, _horizontal = value);
		}
		
		[Bindable(event="verticalChange")]
		public function get vertical():IPosition { return _vertical; }
		public function set vertical(value:IPosition):void {
			if(_vertical == value) {
				return;
			}
			PropertyEvent.dispatchChange(this, "vertical", _vertical, _vertical = value);
		}
		
		[Bindable(event="containerChange")]
		public function get container():DisplayObject { return _container; }
		public function set container(value:DisplayObject):void {
			if(_container) { this.removeChild(_container); }
			var oldContainer:DisplayObject = _container;
			_container = value;
			if(_container) { this.addChild(_container); }
			PropertyEvent.dispatchChange(this, "container", oldContainer, _container);
		}
		
		public function ScrollDisplay()
		{
			horizontal = new Position();
			vertical = new Position();
			reflex.metadata.resolveCommitProperties(this);
			Bind.addListener(this, onHorizontalScroll, this, "horizontal.position");
			Bind.addListener(this, onVerticalScroll, this, "vertical.position");
		}
		
		/**
		 * @private
		 **/
		// If we keep position by percent, what happens when the scrolled object's size changes? (That's a rhetorical question.)
		[CommitProperties(target="container, vertical, horizontal, container.height, container.width")]
		public function onContainerChange(event:Event):void {
			horizontal.min = vertical.min = 0;
			measured.width = horizontal.max = reflex.measurement.resolveWidth(container);
			measured.height = vertical.max = reflex.measurement.resolveHeight(container);
			if(horizontal is IScroll) {
				(horizontal as IScroll).pageSize = unscaledWidth;
			}
			if(vertical is IScroll) {
				(vertical as IScroll).pageSize = unscaledHeight;
			}
			shape = new Shape();
			shape.graphics.beginFill(0, 0);
			shape.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
			shape.graphics.endFill();
			this.addChild(shape);
			this.mask = shape;
		}
		
		private function onVerticalScroll(value:Object):void {
			if(vertical && container) {
				var height:Number = reflex.measurement.resolveHeight(container);
				var potential:Number = height - unscaledHeight;
				var percent:Number = (vertical.position-vertical.min) /(vertical.max-vertical.min);
				container.y = potential * percent * -1; // ui specific calcs are here, where the ui specific code is.
			}
		}
		
		private function onHorizontalScroll(value:Object):void {
			if(horizontal && container) {
				var width:Number = reflex.measurement.resolveWidth(container);
				var potential:Number = width - unscaledWidth;
				var percent:Number = (horizontal.position-horizontal.min) /(horizontal.max-horizontal.min);
				container.x = potential * percent * -1; // ui specific calcs are here, where the ui specific code is.
			}
		}
		
	}
}